import json
import os
from .CloudApiBase import CloudApiBase


class PeopleBusinessSearch(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://search.melissadata.net", "/v5/web/contactsearch/docontactSearch")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.post_body = None

        self.parameter_mappings = {
            "address_line_1": "a1",
            "administrative_area": "adminarea",
            "country": "ctry",
            "locality": "loc",
            "phone": "phone",
            "postal": "postal",
            "post_direction": "postdir",
            "pre_direction": "predir",
            "premises_number": "premnum",
            "sub_premises_number": "subpremnum",
            "thoroughfare_name": "tname",
            "trailing_type": "trailingtype",
            "any_name": "anyname",
            "company": "comp",
            "first_name": "first",
            "full_name": "full",
            "last_name": "last",
            "sub_user": "subuser",
            "transmission_reference": "t",
            "match_level": "matchlevel",
            "max_records": "maxrecords",
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def administrative_area(self):
        return self._administrative_area

    @administrative_area.setter
    def administrative_area(self, value):
        self._set_parameter("administrative_area", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def locality(self):
        return self._locality

    @locality.setter
    def locality(self, value):
        self._set_parameter("locality", value)

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._set_parameter("phone", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def post_direction(self):
        return self._post_direction

    @post_direction.setter
    def post_direction(self, value):
        self._set_parameter("post_direction", value)

    @property
    def pre_direction(self):
        return self._pre_direction

    @pre_direction.setter
    def pre_direction(self, value):
        self._set_parameter("pre_direction", value)

    @property
    def premises_number(self):
        return self._premises_number

    @premises_number.setter
    def premises_number(self, value):
        self._set_parameter("premises_number", value)

    @property
    def sub_premises_number(self):
        return self._sub_premises_number

    @sub_premises_number.setter
    def sub_premises_number(self, value):
        self._set_parameter("sub_premises_number", value)

    @property
    def thoroughfare_name(self):
        return self._thoroughfare_name

    @thoroughfare_name.setter
    def thoroughfare_name(self, value):
        self._set_parameter("thoroughfare_name", value)

    @property
    def trailing_type(self):
        return self._trailing_type

    @trailing_type.setter
    def trailing_type(self, value):
        self._set_parameter("trailing_type", value)

    @property
    def any_name(self):
        return self._any_name

    @any_name.setter
    def any_name(self, value):
        self._set_parameter("any_name", value)

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._set_parameter("company", value)

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._set_parameter("first_name", value)

    @property
    def full_name(self):
        return self._full_name

    @full_name.setter
    def full_name(self, value):
        self._set_parameter("full_name", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def sub_user(self):
        return self._sub_user

    @sub_user.setter
    def sub_user(self, value):
        self._set_parameter("sub_user", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def match_level(self):
        return self._match_level

    @match_level.setter
    def match_level(self, value):
        self._set_parameter("match_level", value)

    @property
    def max_records(self):
        return self._max_records

    @max_records.setter
    def max_records(self, value):
        self._set_parameter("max_records", value)

    # Setters
    def set_address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    def set_administrative_area(self, value):
        self._set_parameter("administrative_area", value)

    def set_country(self, value):
        self._set_parameter("country", value)

    def set_locality(self, value):
        self._set_parameter("locality", value)

    def set_phone(self, value):
        self._set_parameter("phone", value)

    def set_postal(self, value):
        self._set_parameter("postal", value)

    def set_post_direction(self, value):
        self._set_parameter("post_direction", value)

    def set_pre_direction(self, value):
        self._set_parameter("pre_direction", value)

    def set_premises_number(self, value):
        self._set_parameter("premises_number", value)

    def set_sub_premises_number(self, value):
        self._set_parameter("sub_premises_number", value)

    def set_thoroughfare_name(self, value):
        self._set_parameter("thoroughfare_name", value)

    def set_trailing_type(self, value):
        self._set_parameter("trailing_type", value)

    def set_any_name(self, value):
        self._set_parameter("any_name", value)

    def set_company(self, value):
        self._set_parameter("company", value)

    def set_first_name(self, value):
        self._set_parameter("first_name", value)

    def set_full_name(self, value):
        self._set_parameter("full_name", value)

    def set_last_name(self, value):
        self._set_parameter("last_name", value)

    def set_sub_user(self, value):
        self._set_parameter("sub_user", value)

    def set_transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    def set_match_level(self, value):
        self._set_parameter("match_level", value)

    def set_max_records(self, value):
        self._set_parameter("max_records", value)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None

        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]

            property_name = next((key for key, val in self.parameter_mappings.items()
                                  if val.lower() == parameter_key.lower()), None)

            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_address_line_1(self):
        return self.parameters.get("a1") or ""

    def get_administrative_area(self):
        return self.parameters.get("adminarea") or ""

    def get_country(self):
        return self.parameters.get("ctry") or ""

    def get_locality(self):
        return self.parameters.get("loc") or ""

    def get_phone(self):
        return self.parameters.get("phone") or ""

    def get_postal(self):
        return self.parameters.get("postal") or ""

    def get_post_direction(self):
        return self.parameters.get("postdir") or ""

    def get_pre_direction(self):
        return self.parameters.get("predir") or ""

    def get_premises_number(self):
        return self.parameters.get("premnum") or ""

    def get_sub_premises_number(self):
        return self.parameters.get("subpremnum") or ""

    def get_thoroughfare_name(self):
        return self.parameters.get("tname") or ""

    def get_trailing_type(self):
        return self.parameters.get("trailingtype") or ""

    def get_any_name(self):
        return self.parameters.get("anyname") or ""

    def get_company(self):
        return self.parameters.get("comp") or ""

    def get_first_name(self):
        return self.parameters.get("first") or ""

    def get_full_name(self):
        return self.parameters.get("full") or ""

    def get_last_name(self):
        return self.parameters.get("last") or ""

    def get_sub_user(self):
        return self.parameters.get("subuser") or ""

    def get_transmission_reference(self):
        return self.parameters.get("t") or ""

    def get_match_level(self):
        return self.parameters.get("matchlevel") or ""

    def get_max_records(self):
        return self.parameters.get("maxrecords") or ""

    def get_post_body(self):
        return self.post_body
    
    # Class Methods
    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)


    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version





    


class BusinessCoderRecordRequest:
    def __init__(self, a1=None, a2=None, city=None, comp=None, ctry=None, freeform=None, mak=None, mek=None, phone=None, 
                 postal=None, rec=None, state=None, stock=None, suite=None, web=None):
        self.a1 = a1
        self.a2 = a2
        self.city = city
        self.comp = comp
        self.ctry = ctry
        self.freeform = freeform
        self.mak = mak
        self.mek = mek
        self.phone = phone
        self.postal = postal
        self.rec = rec
        self.state = state
        self.stock = stock
        self.suite = suite
        self.web = web

    def to_dict(self):
        return {
            "A1": self.a1,
            "A2": self.a2,
            "City": self.city,
            "Comp": self.comp,
            "Ctry": self.ctry,
            "Freeform": self.freeform,
            "Mak": self.mak,
            "Mek": self.mek,
            "Phone": self.phone,
            "Postal": self.postal,
            "Rec": self.rec,
            "State": self.state,
            "Stock": self.stock,
            "Suite": self.suite,
            "Web": self.web
        }

class GlobalAddressVerificationRecordRequest:
    def __init__(self, record_id=None, organization=None, address_line_1=None, address_line_2=None, 
                 address_line_3=None, address_line_4=None, address_line_5=None, address_line_6=None, 
                 address_line_7=None, address_line_8=None, double_dependent_locality=None, 
                 dependent_locality=None, locality=None, sub_administrative_area=None, 
                 administrative_area=None, postal_code=None, sub_national_area=None, country=None):
        
        self.record_id = record_id
        self.organization = organization
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_line_3 = address_line_3
        self.address_line_4 = address_line_4
        self.address_line_5 = address_line_5
        self.address_line_6 = address_line_6
        self.address_line_7 = address_line_7
        self.address_line_8 = address_line_8
        self.double_dependent_locality = double_dependent_locality
        self.dependent_locality = dependent_locality
        self.locality = locality
        self.sub_administrative_area = sub_administrative_area
        self.administrative_area = administrative_area
        self.postal_code = postal_code
        self.sub_national_area = sub_national_area
        self.country = country

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "Organization": self.organization,
            "AddressLine1": self.address_line_1,
            "AddressLine2": self.address_line_2,
            "AddressLine3": self.address_line_3,
            "AddressLine4": self.address_line_4,
            "AddressLine5": self.address_line_5,
            "AddressLine6": self.address_line_6,
            "AddressLine7": self.address_line_7,
            "AddressLine8": self.address_line_8,
            "DoubleDependentLocality": self.double_dependent_locality,
            "DependentLocality": self.dependent_locality,
            "Locality": self.locality,
            "SubAdministrativeArea": self.sub_administrative_area,
            "AdministrativeArea": self.administrative_area,
            "PostalCode": self.postal_code,
            "SubNationalArea": self.sub_national_area,
            "Country": self.country,
        }

    
class GlobalEmailRecordRequest:
    def __init__(self, record_id=None, email=None):
        self.record_id = record_id
        self.email = email

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "Email": self.email
        }
    
class GlobalIPRecordRequest:
    def __init__(self, record_id=None, ip_address=None):
        self.record_id = record_id
        self.ip_address = ip_address

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "IPAddress": self.ip_address
        }

class GlobalNameRecordRequest:
    def __init__(self, record_id=None, full_name=None, company=None):
        self.record_id = record_id
        self.full_name = full_name
        self.company = company

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "FullName": self.full_name,
            "Company": self.company
        }
    
class GlobalPhoneRecordRequest:
    def __init__(self, record_id=None, phone_number=None, country=None, country_of_origin=None):
        self.record_id = record_id
        self.phone_number = phone_number
        self.country = country
        self.country_of_origin = country_of_origin

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "PhoneNumber": self.phone_number,
            "Country": self.country,
            "CountryOfOrigin": self.country_of_origin
        }
    

class PeopleBusinessSearchRecordRequest:
    def __init__(self, customer_id=None, match_level=None, max_records=None, transmission_reference=None,
                 address_line_1=None, administrative_area=None, country=None, locality=None, phone_number=None,
                 postal_code=None, premise_number=None, subpremises_number=None, thoroughfare_name=None,
                 thoroughfare_postdirection=None, thoroughfare_predirection=None, thoroughfare_trailing_type=None,
                 any_name=None, company_name=None, first_name=None, full_name=None, last_name=None, sub_user=None):
        self.customer_id = customer_id
        self.match_level = match_level
        self.max_records = max_records
        self.transmission_reference = transmission_reference

        # Address Parameters
        self.address_line_1 = address_line_1
        self.administrative_area = administrative_area
        self.country = country
        self.locality = locality
        self.phone_number = phone_number
        self.postal_code = postal_code
        self.premise_number = premise_number
        self.subpremises_number = subpremises_number
        self.thoroughfare_name = thoroughfare_name
        self.thoroughfare_postdirection = thoroughfare_postdirection
        self.thoroughfare_predirection = thoroughfare_predirection
        self.thoroughfare_trailing_type = thoroughfare_trailing_type

        # Name Parameters
        self.any_name = any_name
        self.company_name = company_name
        self.first_name = first_name
        self.full_name = full_name
        self.last_name = last_name
        self.sub_user = sub_user

    def to_dict(self):
        return {
            "CustomerID": self.customer_id,
            "MatchLevel": self.match_level,
            "MaxRecords": self.max_records,
            "TransmissionReference": self.transmission_reference,
            
            # Address Parameters
            "AddressLine1": self.address_line_1,
            "AdministrativeArea": self.administrative_area,
            "Country": self.country,
            "Locality": self.locality,
            "PhoneNumber": self.phone_number,
            "PostalCode": self.postal_code,
            "PremiseNumber": self.premise_number,
            "SubpremisesNumber": self.subpremises_number,
            "ThoroughfareName": self.thoroughfare_name,
            "ThoroughfarePostdirection": self.thoroughfare_postdirection,
            "ThoroughfarePredirection": self.thoroughfare_predirection,
            "ThoroughfareTrailingType": self.thoroughfare_trailing_type,
            
            # Name Parameters
            "AnyName": self.any_name,
            "CompanyName": self.company_name,
            "FirstName": self.first_name,
            "FullName": self.full_name,
            "LastName": self.last_name,
            "SubUser": self.sub_user
        }


class PersonatorConsumerRecordRequest:
    def __init__(self, record_id=None, address_line_1=None, address_line_2=None, birth_day=None, birth_month=None,
                 birth_year=None, city=None, company_name=None, country=None, email_address=None, first_name=None,
                 free_form=None, full_name=None, ip_address=None, last_line=None, last_name=None, melissa_address_key=None,
                 phone_number=None, postal_code=None, social_security=None, state=None):
        self.record_id = record_id
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.birth_day = birth_day
        self.birth_month = birth_month
        self.birth_year = birth_year
        self.city = city
        self.company_name = company_name
        self.country = country
        self.email_address = email_address
        self.first_name = first_name
        self.free_form = free_form
        self.full_name = full_name
        self.ip_address = ip_address
        self.last_line = last_line
        self.last_name = last_name
        self.melissa_address_key = melissa_address_key
        self.phone_number = phone_number
        self.postal_code = postal_code
        self.social_security = social_security
        self.state = state

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "AddressLine1": self.address_line_1,
            "AddressLine2": self.address_line_2,
            "BirthDay": self.birth_day,
            "BirthMonth": self.birth_month,
            "BirthYear": self.birth_year,
            "City": self.city,
            "CompanyName": self.company_name,
            "Country": self.country,
            "EmailAddress": self.email_address,
            "FirstName": self.first_name,
            "FreeForm": self.free_form,
            "FullName": self.full_name,
            "IPAddress": self.ip_address,
            "LastLine": self.last_line,
            "LastName": self.last_name,
            "MelissaAddressKey": self.melissa_address_key,
            "PhoneNumber": self.phone_number,
            "PostalCode": self.postal_code,
            "SocialSecurity": self.social_security,
            "State": self.state
        }

class PersonatorIdentityRecordRequest():
    def __init__(
        self, transmission_reference="", customer_id="", actions="", address_options="", email_options="", phone_options="", 
        name_options="", options="", national_id="", first_name="", last_name="", full_name="", company="", phone_number="", 
        email="", date_of_birth="", account_number="", address_line_1="", address_line_2="", address_line_3="", address_line_4="", 
        address_line_5="", address_line_6="", address_line_7="", address_line_8="", locality="", administrative_area="", 
        postal_code="", country="", country_of_origin="", state=""
    ):
        self.transmission_reference = transmission_reference
        self.customer_id = customer_id
        self.actions = actions
        self.address_options = address_options
        self.email_options = email_options
        self.phone_options = phone_options
        self.name_options = name_options
        self.options = options
        self.national_id = national_id
        self.first_name = first_name
        self.last_name = last_name
        self.full_name = full_name
        self.company = company
        self.phone_number = phone_number
        self.email = email
        self.date_of_birth = date_of_birth
        self.account_number = account_number
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_line_3 = address_line_3
        self.address_line_4 = address_line_4
        self.address_line_5 = address_line_5
        self.address_line_6 = address_line_6
        self.address_line_7 = address_line_7
        self.address_line_8 = address_line_8
        self.locality = locality
        self.administrative_area = administrative_area
        self.postal_code = postal_code
        self.country = country
        self.country_of_origin = country_of_origin
        self.state = state

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "CustomerID": self.customer_id,
            "Actions": self.actions,
            "AddressOptions": self.address_options,
            "EmailOptions": self.email_options,
            "PhoneOptions": self.phone_options,
            "NameOptions": self.name_options,
            "Options": self.options,
            "NationalID": self.national_id,
            "FirstName": self.first_name,
            "LastName": self.last_name,
            "FullName": self.full_name,
            "Company": self.company,
            "PhoneNumber": self.phone_number,
            "Email": self.email,
            "DateOfBirth": self.date_of_birth,
            "AccountNumber": self.account_number,
            "AddressLine1": self.address_line_1,
            "AddressLine2": self.address_line_2,
            "AddressLine3": self.address_line_3,
            "AddressLine4": self.address_line_4,
            "AddressLine5": self.address_line_5,
            "AddressLine6": self.address_line_6,
            "AddressLine7": self.address_line_7,
            "AddressLine8": self.address_line_8,
            "Locality": self.locality,
            "AdministrativeArea": self.administrative_area,
            "PostalCode": self.postal_code,
            "Country": self.country,
            "CountryOfOrigin": self.country_of_origin,
            "State": self.state,
        }

class PropertyRecordRequest:
    def __init__(
        self, 
        transmission_reference="", customer_id="", columns="", options="",
        record_id="", account="", address_key="", address_line_1="", 
        address_line_2="", apn="", city="", country="", fips="", 
        free_form="", property_mak="", owner_mak="", mak="", 
        state="", postal_code="", transaction_id=""
    ):
        self.transmission_reference = transmission_reference
        self.customer_id = customer_id
        self.columns = columns
        self.options = options
        self.record_id = record_id
        self.account = account
        self.address_key = address_key
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.apn = apn
        self.city = city
        self.country = country
        self.fips = fips
        self.free_form = free_form
        self.property_mak = property_mak
        self.owner_mak = owner_mak
        self.mak = mak
        self.state = state
        self.postal_code = postal_code
        self.transaction_id = transaction_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "CustomerId": self.customer_id,
            "Columns": self.columns,
            "Options": self.options,
            "RecordId": self.record_id,
            "Account": self.account,
            "AddressKey": self.address_key,
            "AddressLine1": self.address_line_1,
            "AddressLine2": self.address_line_2,
            "APN": self.apn,
            "City": self.city,
            "Country": self.country,
            "FIPS": self.fips,
            "FreeForm": self.free_form,
            "PropertyMAK": self.property_mak,
            "OwnerMAK": self.owner_mak,
            "MAK": self.mak,
            "State": self.state,
            "PostalCode": self.postal_code,
            "TransactionID": self.transaction_id
        }


class ReverseGeoCoderRecordRequest:
    def __init__(
        self,
        customer_id="",
        latitude="",
        longitude="",
        max_distance="",
        max_records="",
        options="",
        transmission_reference="",
    ):
        self.customer_id = customer_id
        self.latitude = latitude
        self.longitude = longitude
        self.max_distance = max_distance
        self.max_records = max_records
        self.options = options
        self.transmission_reference = transmission_reference

    def to_dict(self):
        return {
            "CustomerId": self.customer_id,
            "Latitude": self.latitude,
            "Longitude": self.longitude,
            "MaxDistance": self.max_distance,
            "MaxRecords": self.max_records,
            "Options": self.options,
            "TransmissionReference": self.transmission_reference,
        }

class SmartMoverRecordRequest:
    def __init__(self, record_id="", company="", name_full="", name_first="", name_middle="",
                 name_prefix="", name_suffix="", name_last="", urbanization="", address_line_1="",
                 address_line_2="", suite="", private_mailbox="", city="", state="", postal_code="",
                 plus4="", country=""):
        self.record_id = record_id
        self.company = company
        self.name_full = name_full
        self.name_first = name_first
        self.name_middle = name_middle
        self.name_prefix = name_prefix
        self.name_suffix = name_suffix
        self.name_last = name_last
        self.urbanization = urbanization
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.suite = suite
        self.private_mailbox = private_mailbox
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.plus4 = plus4
        self.country = country

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "Company": self.company,
            "NameFull": self.name_full,
            "NameFirst": self.name_first,
            "NameMiddle": self.name_middle,
            "NamePrefix": self.name_prefix,
            "NameSuffix": self.name_suffix,
            "NameLast": self.name_last,
            "Urbanization": self.urbanization,
            "AddressLine1": self.address_line_1,
            "AddressLine2": self.address_line_2,
            "Suite": self.suite,
            "PrivateMailbox": self.private_mailbox,
            "City": self.city,
            "State": self.state,
            "PostalCode": self.postal_code,
            "Plus4": self.plus4,
            "Country": self.country
        }
    
class SSNNameMatchRecordRequest:
    def __init__(
        self,
        record_id="",
        ssn="",
        first_name="",
        last_name="",
        full_name=""
    ):
        self.record_id = record_id
        self.ssn = ssn
        self.first_name = first_name
        self.last_name = last_name
        self.full_name = full_name

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "SSN": self.ssn,
            "FirstName": self.first_name,
            "LastName": self.last_name,
            "FullName": self.full_name
        }
    
class StreetRouteRecordRequest:
    def __init__(self, record_id="", start_latitude="", start_longitude="", end_latitude="", end_longitude=""):
        self.record_id = record_id
        self.start_latitude = start_latitude
        self.start_longitude = start_longitude
        self.end_latitude = end_latitude
        self.end_longitude = end_longitude

    def to_dict(self):
        return {
            "RecordID": self.record_id,
            "StartLatitude": self.start_latitude,
            "StartLongitude": self.start_longitude,
            "EndLatitude": self.end_latitude,
            "EndLongitude": self.end_longitude
        }






import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import SmartMoverPostRequest

class SmartMover(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://smartmover.melissadata.net", "/V3/WEB/SmartMover/doSmartMover")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "job_id": "jobid",
            "paf_id": "pafid",
            "action": "act",
            "list": "list",
            "company": "comp",
            "full_name": "full",
            "first_name": "first",
            "middle_name": "middle",
            "name_prefix": "namepre",
            "name_suffix": "namesfx",
            "last_name": "last",
            "urbanization": "u",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "suite": "ste",
            "private_mailbox": "pmb",
            "city": "city",
            "state": "state",
            "postal": "postal",
            "plus4": "plus4",
            "country": "ctry",
            "transmission_reference": "t",
            "opt": "opt",
            "cols": "cols"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def job_id(self):
        return self._job_id

    @job_id.setter
    def job_id(self, value):
        self._set_parameter("job_id", value)

    @property
    def paf_id(self):
        return self._paf_id

    @paf_id.setter
    def paf_id(self, value):
        self._set_parameter("paf_id", value)

    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, value):
        self._set_parameter("action", value)

    @property
    def list(self):
        return self._list

    @list.setter
    def list(self, value):
        self._set_parameter("list", value)

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._set_parameter("company", value)

    @property
    def full_name(self):
        return self._full_name

    @full_name.setter
    def full_name(self, value):
        self._set_parameter("full_name", value)

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._set_parameter("first_name", value)

    @property
    def middle_name(self):
        return self._middle_name

    @middle_name.setter
    def middle_name(self, value):
        self._set_parameter("middle_name", value)

    @property
    def name_prefix(self):
        return self._name_prefix

    @name_prefix.setter
    def name_prefix(self, value):
        self._set_parameter("name_prefix", value)

    @property
    def name_suffix(self):
        return self._name_suffix

    @name_suffix.setter
    def name_suffix(self, value):
        self._set_parameter("name_suffix", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def urbanization(self):
        return self._urbanization

    @urbanization.setter
    def urbanization(self, value):
        self._set_parameter("urbanization", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def suite(self):
        return self._suite

    @suite.setter
    def suite(self, value):
        self._set_parameter("suite", value)

    @property
    def private_mailbox(self):
        return self._private_mailbox

    @private_mailbox.setter
    def private_mailbox(self, value):
        self._set_parameter("private_mailbox", value)

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._set_parameter("city", value)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._set_parameter("state", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def plus4(self):
        return self._plus4

    @plus4.setter
    def plus4(self, value):
        self._set_parameter("plus4", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def cols(self):
        return self._cols

    @cols.setter
    def cols(self, value):
        self._set_parameter("cols", value)

    # Setters
    def set_job_id(self, job_id):
        self._set_parameter("job_id", job_id)

    def set_paf_id(self, paf_id):
        self._set_parameter("paf_id", paf_id)

    def set_action(self, action):
        self._set_parameter("action", action)

    def set_list(self, list_):
        self._set_parameter("list", list_)

    def set_company(self, company):
        self._set_parameter("company", company)

    def set_full_name(self, full_name):
        self._set_parameter("full_name", full_name)

    def set_first_name(self, first_name):
        self._set_parameter("first_name", first_name)

    def set_middle_name(self, middle_name):
        self._set_parameter("middle_name", middle_name)

    def set_name_prefix(self, name_prefix):
        self._set_parameter("name_prefix", name_prefix)

    def set_name_suffix(self, name_suffix):
        self._set_parameter("name_suffix", name_suffix)

    def set_last_name(self, last_name):
        self._set_parameter("last_name", last_name)

    def set_urbanization(self, urbanization):
        self._set_parameter("urbanization", urbanization)

    def set_address_line_1(self, address_line_1):
        self._set_parameter("address_line_1", address_line_1)

    def set_address_line_2(self, address_line_2):
        self._set_parameter("address_line_2", address_line_2)

    def set_suite(self, suite):
        self._set_parameter("suite", suite)

    def set_private_mailbox(self, private_mailbox):
        self._set_parameter("private_mailbox", private_mailbox)

    def set_city(self, city):
        self._set_parameter("city", city)

    def set_state(self, state):
        self._set_parameter("state", state)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_plus4(self, plus4):
        self._set_parameter("plus4", plus4)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_cols(self, cols):
        self._set_parameter("cols", cols)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_job_id(self):
        return self.parameters.get("jobid", "")

    def get_paf_id(self):
        return self.parameters.get("pafid", "")

    def get_action(self):
        return self.parameters.get("act", "")

    def get_list(self):
        return self.parameters.get("list", "")

    def get_company(self):
        return self.parameters.get("comp", "")

    def get_full_name(self):
        return self.parameters.get("full", "")

    def get_first_name(self):
        return self.parameters.get("first", "")

    def get_middle_name(self):
        return self.parameters.get("middle", "")

    def get_name_prefix(self):
        return self.parameters.get("namepre", "")

    def get_name_suffix(self):
        return self.parameters.get("namesfx", "")

    def get_last_name(self):
        return self.parameters.get("last", "")

    def get_urbanization(self):
        return self.parameters.get("u", "")

    def get_address_line_1(self):
        return self.parameters.get("a1", "")

    def get_address_line_2(self):
        return self.parameters.get("a2", "")

    def get_suite(self):
        return self.parameters.get("ste", "")

    def get_private_mailbox(self):
        return self.parameters.get("pmb", "")

    def get_city(self):
        return self.parameters.get("city", "")

    def get_state(self):
        return self.parameters.get("state", "")

    def get_postal(self):
        return self.parameters.get("postal", "")

    def get_plus_4(self):
        return self.parameters.get("plus4", "")

    def get_country(self):
        return self.parameters.get("ctry", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_cols(self):
        return self.parameters.get("cols", "")

    def get_post_body(self):
        return self.post_body
    
    # Class Methods
    def add_record(self, request):
        self.records.append(request)
    
    def clear_records(self):
        self.records.clear()

    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """

        if self.records and len(self.records) > 0:
            post_request = SmartMoverPostRequest(
                transmission_reference=self.get_transmission_reference(),
                records=self.records,
                customer_id=self.get_license(),
                job_id=self.get_job_id(),
                paf_id=self.get_paf_id(),
                actions=self.get_action(),
                options=self.get_opt(),
                columns=self.get_cols(),
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)

    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version




from .ResponseBase import ResponseBase

class GlobalIPResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", transmission_results="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        records = [GlobalIPRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            records=records
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""

class GlobalIPRecord(ResponseBase):
    def __init__(self, city="", connection_speed="", connection_type="", continent="", country_abbreviation="", 
                 country_name="", domain_address1="", domain_administrative_area="", domain_age_estimated="", 
                 domain_availability="", domain_country="", domain_country_code="", domain_created_date="", 
                 domain_email="", domain_expiration_date="", domain_locality="", domain_name="", domain_organization="", 
                 domain_postal_code="", domain_private_proxy="", domain_updated_date="", dst="", ip_address="", 
                 isp_name="", latitude="", longitude="", postal_code="", proxy_description="", proxy_type="", 
                 record_id="", region="", result="", time_zone_code="", time_zone_name="", utc=""):
        self.city = city
        self.connection_speed = connection_speed
        self.connection_type = connection_type
        self.continent = continent
        self.country_abbreviation = country_abbreviation
        self.country_name = country_name
        self.domain_address1 = domain_address1
        self.domain_administrative_area = domain_administrative_area
        self.domain_age_estimated = domain_age_estimated
        self.domain_availability = domain_availability
        self.domain_country = domain_country
        self.domain_country_code = domain_country_code
        self.domain_created_date = domain_created_date
        self.domain_email = domain_email
        self.domain_expiration_date = domain_expiration_date
        self.domain_locality = domain_locality
        self.domain_name = domain_name
        self.domain_organization = domain_organization
        self.domain_postal_code = domain_postal_code
        self.domain_private_proxy = domain_private_proxy
        self.domain_updated_date = domain_updated_date
        self.dst = dst
        self.ip_address = ip_address
        self.isp_name = isp_name
        self.latitude = latitude
        self.longitude = longitude
        self.postal_code = postal_code
        self.proxy_description = proxy_description
        self.proxy_type = proxy_type
        self.record_id = record_id
        self.region = region
        self.result = result
        self.time_zone_code = time_zone_code
        self.time_zone_name = time_zone_name
        self.utc = utc

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            city=data.get("City", ""),
            connection_speed=data.get("ConnectionSpeed", ""),
            connection_type=data.get("ConnectionType", ""),
            continent=data.get("Continent", ""),
            country_abbreviation=data.get("CountryAbbreviation", ""),
            country_name=data.get("CountryName", ""),
            domain_address1=data.get("DomainAddress1", ""),
            domain_administrative_area=data.get("DomainAdministrativeArea", ""),
            domain_age_estimated=data.get("DomainAgeEstimated", ""),
            domain_availability=data.get("DomainAvailability", ""),
            domain_country=data.get("DomainCountry", ""),
            domain_country_code=data.get("DomainCountryCode", ""),
            domain_created_date=data.get("DomainCreatedDate", ""),
            domain_email=data.get("DomainEmail", ""),
            domain_expiration_date=data.get("DomainExpirationDate", ""),
            domain_locality=data.get("DomainLocality", ""),
            domain_name=data.get("DomainName", ""),
            domain_organization=data.get("DomainOrganization", ""),
            domain_postal_code=data.get("DomainPostalCode", ""),
            domain_private_proxy=data.get("DomainPrivateProxy", ""),
            domain_updated_date=data.get("DomainUpdatedDate", ""),
            dst=data.get("DST", ""),
            ip_address=data.get("IPAddress", ""),
            isp_name=data.get("ISPName", ""),
            latitude=data.get("Latitude", ""),
            longitude=data.get("Longitude", ""),
            postal_code=data.get("PostalCode", ""),
            proxy_description=data.get("ProxyDescription", ""),
            proxy_type=data.get("ProxyType", ""),
            record_id=data.get("RecordID", ""),
            region=data.get("Region", ""),
            result=data.get("Result", ""),
            time_zone_code=data.get("TimeZoneCode", ""),
            time_zone_name=data.get("TimeZoneName", ""),
            utc=data.get("UTC", "")
        )

    # Setters
    def set_city(self, city):
        self.city = city

    def set_connection_speed(self, connection_speed):
        self.connection_speed = connection_speed

    def set_connection_type(self, connection_type):
        self.connection_type = connection_type

    def set_continent(self, continent):
        self.continent = continent

    def set_country_abbreviation(self, country_abbreviation):
        self.country_abbreviation = country_abbreviation

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_domain_address1(self, domain_address1):
        self.domain_address1 = domain_address1

    def set_domain_administrative_area(self, domain_administrative_area):
        self.domain_administrative_area = domain_administrative_area

    def set_domain_age_estimated(self, domain_age_estimated):
        self.domain_age_estimated = domain_age_estimated

    def set_domain_availability(self, domain_availability):
        self.domain_availability = domain_availability

    def set_domain_country(self, domain_country):
        self.domain_country = domain_country

    def set_domain_country_code(self, domain_country_code):
        self.domain_country_code = domain_country_code

    def set_domain_created_date(self, domain_created_date):
        self.domain_created_date = domain_created_date

    def set_domain_email(self, domain_email):
        self.domain_email = domain_email

    def set_domain_expiration_date(self, domain_expiration_date):
        self.domain_expiration_date = domain_expiration_date

    def set_domain_locality(self, domain_locality):
        self.domain_locality = domain_locality

    def set_domain_name(self, domain_name):
        self.domain_name = domain_name

    def set_domain_organization(self, domain_organization):
        self.domain_organization = domain_organization

    def set_domain_postal_code(self, domain_postal_code):
        self.domain_postal_code = domain_postal_code

    def set_domain_private_proxy(self, domain_private_proxy):
        self.domain_private_proxy = domain_private_proxy

    def set_domain_updated_date(self, domain_updated_date):
        self.domain_updated_date = domain_updated_date

    def set_dst(self, dst):
        self.dst = dst

    def set_ip_address(self, ip_address):
        self.ip_address = ip_address

    def set_isp_name(self, isp_name):
        self.isp_name = isp_name

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_proxy_description(self, proxy_description):
        self.proxy_description = proxy_description

    def set_proxy_type(self, proxy_type):
        self.proxy_type = proxy_type

    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_region(self, region):
        self.region = region

    def set_result(self, result):
        self.result = result

    def set_time_zone_code(self, time_zone_code):
        self.time_zone_code = time_zone_code

    def set_time_zone_name(self, time_zone_name):
        self.time_zone_name = time_zone_name

    def set_utc(self, utc):
        self.utc = utc

    # Getters
    def get_city(self):
        return self.city or ""

    def get_connection_speed(self):
        return self.connection_speed or ""

    def get_connection_type(self):
        return self.connection_type or ""

    def get_continent(self):
        return self.continent or ""

    def get_country_abbreviation(self):
        return self.country_abbreviation or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_domain_address1(self):
        return self.domain_address1 or ""

    def get_domain_administrative_area(self):
        return self.domain_administrative_area or ""

    def get_domain_age_estimated(self):
        return self.domain_age_estimated or ""

    def get_domain_availability(self):
        return self.domain_availability or ""

    def get_domain_country(self):
        return self.domain_country or ""

    def get_domain_country_code(self):
        return self.domain_country_code or ""

    def get_domain_created_date(self):
        return self.domain_created_date or ""

    def get_domain_email(self):
        return self.domain_email or ""

    def get_domain_expiration_date(self):
        return self.domain_expiration_date or ""

    def get_domain_locality(self):
        return self.domain_locality or ""

    def get_domain_name(self):
        return self.domain_name or ""

    def get_domain_organization(self):
        return self.domain_organization or ""

    def get_domain_postal_code(self):
        return self.domain_postal_code or ""

    def get_domain_private_proxy(self):
        return self.domain_private_proxy or ""

    def get_domain_updated_date(self):
        return self.domain_updated_date or ""

    def get_dst(self):
        return self.dst or ""

    def get_ip_address(self):
        return self.ip_address or ""

    def get_isp_name(self):
        return self.isp_name or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_proxy_description(self):
        return self.proxy_description or ""

    def get_proxy_type(self):
        return self.proxy_type or ""

    def get_record_id(self):
        return self.record_id or ""

    def get_region(self):
        return self.region or ""

    def get_result(self):
        return self.result or ""

    def get_time_zone_code(self):
        return self.time_zone_code or ""

    def get_time_zone_name(self):
        return self.time_zone_name or ""

    def get_utc(self):
        return self.utc or ""


import requests
import json
from urllib.parse import urlencode, quote
from abc import ABC

class CloudApiBase(ABC):
    """
    CloudApiBase sets the foundation of every CloudApi class.
    This class contains variables and methods that all CloudApi classes share.
    This class contains all the methods that are responsible for sending out API requests.
    """

    def __init__(self, base_url=None, endpoint=None):
        self.base_url = base_url
        self.license = None
        self.endpoint = endpoint
        self.parameters = {}

    # Setters
    def set_base_url(self, base_url):
        self.base_url = base_url

    def set_license(self, license):
        self.license = license

    def set_endpoint(self, endpoint):
        self.endpoint = endpoint

    def set_format(self, format):
        self.parameters["format"] = format

    # Getters
    def get_base_url(self):
        return self.base_url or ""

    def get_license(self):
        return self.license or ""

    def get_endpoint(self):
        return self.endpoint or ""

    def get_format(self):
        return self.parameters.get("format", "")

    def get_value(self, parameter):
        if parameter.lower() == "baseurl":
            return self.base_url
        elif parameter.lower() == "license":
            return self.base_url
        elif parameter.lower() == "endpoint":
            return self.base_url
        elif parameter.lower() == "format":
            return self.base_url
        
        return self.parameters.get(parameter, "")

    # Class Methods
    def clear(self):
        """
        This function clears all the variables set for the CloudAPI but remembers the license, baseUrl, and endpoint.
        """
        self.parameters.clear()
        self.parameters["format"] = "json"

    def get_contents(self, base_url, request_query):
        """
        This synchronous function makes the API request using the baseURL and assembled endpoint with parameters (requestQuery).
        """
        response = requests.get(f"{base_url}{request_query}")
        response.raise_for_status()
        text = response.text
        obj = json.loads(text)
        pretty_response = json.dumps(obj, indent=4)
        return pretty_response

    def process_request(self, parameters):
        """
        This synchronous function assembles the endpoint with the parameters and calls GetContents() to make the API request.
        """
        rest_request = "&id=" + quote(self.license)
        rest_request += "&" + urlencode(parameters)
        rest_request = self.endpoint + "?" + rest_request
        response = self.get_contents(self.base_url, rest_request)
        return response

    def send_post_request(self, post_request):
        """
        This synchronous function sends a batch request to the API.
        """
        try:
            headers = {'Content-Type': 'application/json'}
            json_batch = json.dumps(post_request.to_dict())  # Convert to dict first
            response = requests.post(f"{self.base_url}{self.endpoint}", headers=headers, data=json_batch)
            response.raise_for_status()
            response_content = response.text
            
            obj = json.loads(response_content)
            pretty_response = json.dumps(obj, indent=4)

            return pretty_response
        except Exception as ex:
            return str(ex)


    def get(self, response_type=None):
        """
        This function calls ProcessRequest() to send the CloudAPI request and get the response.
        This function coordinates a string or deserialized response object return.
        """
        response = self.process_request(self.parameters)
        return self.handle_response(response, response_type)


    def send_version_request(self):
        """
        This synchronous function makes a request to the getversion endpoint.
        """
        try:
            trimmed_endpoint = self.endpoint.rstrip('/')
            segments = trimmed_endpoint.split('/')
            segments[-1] = "getversion"
            version_endpoint = "/".join(segments)
            response = requests.get(f"{self.base_url}{version_endpoint}")
            response.raise_for_status()
            response_body = response.text
            return response_body
        except Exception as ex:
            return str(ex)


    def handle_response(self, response, response_type):
        """
        This function determines whether to deserialize the json response or not.
        """
        if response_type == str or response_type == None:
            return response
        data = json.loads(response)
        response_objectect = response_type.populate_from_dict(data)

        return response_objectect



from .ResponseBase import ResponseBase

class SmartMoverResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", transmission_results="", total_records="", cass_report_link="", nco_report_link="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.cass_report_link = cass_report_link
        self.nco_report_link = nco_report_link
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        records = [SmartMoverRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records = data.get("TotalRecords", ""),
            cass_report_link = data.get("CASSReportLink", ""),
            nco_report_link = data.get("NCOReportLink", ""),
            records=records
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    def set_cass_report_link(self, cass_report_link):
        self.cass_report_link = cass_report_link

    def set_ncoa_report_link(self, ncoa_report_link):
        self.ncoa_report_link = ncoa_report_link

    # Getters 
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""

    def get_cass_report_link(self):
        return self.cass_report_link or ""

    def get_ncoa_report_link(self):
        return self.ncoa_report_link or ""
    
class SmartMoverRecord:
    def __init__(
        self, address_extras="", address_key="", address_line_1="", address_line_2="", address_type_code="", base_melissa_address_key="", carrier_route="", 
        city="", city_abbreviation="", company_name="", country_code="", country_name="", delivery_indicator="", delivery_point_check_digit="", 
        delivery_point_code="", melissa_address_key="", move_effective_date="", move_type_code="", postal_code="", record_id="", results="", 
        state="", state_name="", urbanization="", name_first="", name_full="", name_last="", name_middle="", name_prefix="", name_suffix="", 
        original_address_line_1="", original_address_line_2="", original_city="", original_city_abbreviation="", original_country_code="", 
        original_plus4="", original_postal_code="", original_private_mailbox="", original_results="", original_state="", original_state_name="", 
        original_suite="", original_urbanization="", address_delivery_installation="", address_house_number="", address_lock_box="", 
        address_post_direction="", address_pre_direction="", address_private_mailbox_name="", address_private_mailbox_range="", 
        address_route_service="", address_street_name="", address_street_suffix="", address_suite_name="", address_suite_number="", 
        standardized_address_line_1="", standardized_address_line_2="", standardized_base_melissa_address_key="", standardized_carrier_route="", 
        standardized_city="", standardized_city_abbreviation="", standardized_country_code="", standardized_delivery_indicator="", 
        standardized_delivery_point_check_digit="", standardized_delivery_point_code="", standardized_melissa_address_key="", 
        standardized_plus4="", standardized_postal_code="", standardized_private_mailbox="", standardized_results="", standardized_state="", 
        standardized_state_name="", standardized_suite="", standardized_urbanization="", dpv_foot_notes="", move_return_code="", plus4="", 
        private_mailbox="", suite="",
    ):
        self.address_extras = address_extras
        self.address_key = address_key
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_type_code = address_type_code
        self.base_melissa_address_key = base_melissa_address_key
        self.carrier_route = carrier_route
        self.city = city
        self.city_abbreviation = city_abbreviation
        self.company_name = company_name
        self.country_code = country_code
        self.country_name = country_name
        self.delivery_indicator = delivery_indicator
        self.delivery_point_check_digit = delivery_point_check_digit
        self.delivery_point_code = delivery_point_code
        self.melissa_address_key = melissa_address_key
        self.move_effective_date = move_effective_date
        self.move_type_code = move_type_code
        self.postal_code = postal_code
        self.record_id = record_id
        self.results = results
        self.state = state
        self.state_name = state_name
        self.urbanization = urbanization
        self.name_first = name_first
        self.name_full = name_full
        self.name_last = name_last
        self.name_middle = name_middle
        self.name_prefix = name_prefix
        self.name_suffix = name_suffix
        self.original_address_line_1 = original_address_line_1
        self.original_address_line_2 = original_address_line_2
        self.original_city = original_city
        self.original_city_abbreviation = original_city_abbreviation
        self.original_country_code = original_country_code
        self.original_plus4 = original_plus4
        self.original_postal_code = original_postal_code
        self.original_private_mailbox = original_private_mailbox
        self.original_results = original_results
        self.original_state = original_state
        self.original_state_name = original_state_name
        self.original_suite = original_suite
        self.original_urbanization = original_urbanization
        self.address_delivery_installation = address_delivery_installation
        self.address_house_number = address_house_number
        self.address_lock_box = address_lock_box
        self.address_post_direction = address_post_direction
        self.address_pre_direction = address_pre_direction
        self.address_private_mailbox_name = address_private_mailbox_name
        self.address_private_mailbox_range = address_private_mailbox_range
        self.address_route_service = address_route_service
        self.address_street_name = address_street_name
        self.address_street_suffix = address_street_suffix
        self.address_suite_name = address_suite_name
        self.address_suite_number = address_suite_number
        self.standardized_address_line_1 = standardized_address_line_1
        self.standardized_address_line_2 = standardized_address_line_2
        self.standardized_base_melissa_address_key = standardized_base_melissa_address_key
        self.standardized_carrier_route = standardized_carrier_route
        self.standardized_city = standardized_city
        self.standardized_city_abbreviation = standardized_city_abbreviation
        self.standardized_country_code = standardized_country_code
        self.standardized_delivery_indicator = standardized_delivery_indicator
        self.standardized_delivery_point_check_digit = standardized_delivery_point_check_digit
        self.standardized_delivery_point_code = standardized_delivery_point_code
        self.standardized_melissa_address_key = standardized_melissa_address_key
        self.standardized_plus4 = standardized_plus4
        self.standardized_postal_code = standardized_postal_code
        self.standardized_private_mailbox = standardized_private_mailbox
        self.standardized_results = standardized_results
        self.standardized_state = standardized_state
        self.standardized_state_name = standardized_state_name
        self.standardized_suite = standardized_suite
        self.standardized_urbanization = standardized_urbanization
        self.dpv_foot_notes = dpv_foot_notes
        self.move_return_code = move_return_code
        self.plus4 = plus4
        self.private_mailbox = private_mailbox
        self.suite = suite

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            address_extras=data.get("AddressExtras", ""),
            address_key=data.get("AddressKey", ""),
            address_line_1=data.get("AddressLine1", ""),
            address_line_2=data.get("AddressLine2", ""),
            address_type_code=data.get("AddressTypeCode", ""),
            base_melissa_address_key=data.get("BaseMelissaAddressKey", ""),
            carrier_route=data.get("CarrierRoute", ""),
            city=data.get("City", ""),
            city_abbreviation=data.get("CityAbbreviation", ""),
            company_name=data.get("CompanyName", ""),
            country_code=data.get("CountryCode", ""),
            country_name=data.get("CountryName", ""),
            delivery_indicator=data.get("DeliveryIndicator", ""),
            delivery_point_check_digit=data.get("DeliveryPointCheckDigit", ""),
            delivery_point_code=data.get("DeliveryPointCode", ""),
            melissa_address_key=data.get("MelissaAddressKey", ""),
            move_effective_date=data.get("MoveEffectiveDate", ""),
            move_type_code=data.get("MoveTypeCode", ""),
            postal_code=data.get("PostalCode", ""),
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            state=data.get("State", ""),
            state_name=data.get("StateName", ""),
            urbanization=data.get("Urbanization", ""),
            name_first=data.get("NameFirst", ""),
            name_full=data.get("NameFull", ""),
            name_last=data.get("NameLast", ""),
            name_middle=data.get("NameMiddle", ""),
            name_prefix=data.get("NamePrefix", ""),
            name_suffix=data.get("NameSuffix", ""),
            original_address_line_1=data.get("OriginalAddressLine1", ""),
            original_address_line_2=data.get("OriginalAddressLine2", ""),
            original_city=data.get("OriginalCity", ""),
            original_city_abbreviation=data.get("OriginalCityAbbreviation", ""),
            original_country_code=data.get("OriginalCountryCode", ""),
            original_plus4=data.get("OriginalPlus4", ""),
            original_postal_code=data.get("OriginalPostalCode", ""),
            original_private_mailbox=data.get("OriginalPrivateMailbox", ""),
            original_results=data.get("OriginalResults", ""),
            original_state=data.get("OriginalState", ""),
            original_state_name=data.get("OriginalStateName", ""),
            original_suite=data.get("OriginalSuite", ""),
            original_urbanization=data.get("OriginalUrbanization", ""),
            address_delivery_installation=data.get("AddressDeliveryInstallation", ""),
            address_house_number=data.get("AddressHouseNumber", ""),
            address_lock_box=data.get("AddressLockBox", ""),
            address_post_direction=data.get("AddressPostDirection", ""),
            address_pre_direction=data.get("AddressPreDirection", ""),
            address_private_mailbox_name=data.get("AddressPrivateMailboxName", ""),
            address_private_mailbox_range=data.get("AddressPrivateMailboxRange", ""),
            address_route_service=data.get("AddressRouteService", ""),
            address_street_name=data.get("AddressStreetName", ""),
            address_street_suffix=data.get("AddressStreetSuffix", ""),
            address_suite_name=data.get("AddressSuiteName", ""),
            address_suite_number=data.get("AddressSuiteNumber", ""),
            standardized_address_line_1=data.get("StandardizedAddressLine1", ""),
            standardized_address_line_2=data.get("StandardizedAddressLine2", ""),
            standardized_base_melissa_address_key=data.get("StandardizedBaseMelissaAddressKey", ""),
            standardized_carrier_route=data.get("StandardizedCarrierRoute", ""),
            standardized_city=data.get("StandardizedCity", ""),
            standardized_city_abbreviation=data.get("StandardizedCityAbbreviation", ""),
            standardized_country_code=data.get("StandardizedCountryCode", ""),
            standardized_delivery_indicator=data.get("StandardizedDeliveryIndicator", ""),
            standardized_delivery_point_check_digit=data.get("StandardizedDeliveryPointCheckDigit", ""),
            standardized_delivery_point_code=data.get("StandardizedDeliveryPointCode", ""),
            standardized_melissa_address_key=data.get("StandardizedMelissaAddressKey", ""),
            standardized_plus4=data.get("StandardizedPlus4", ""),
            standardized_postal_code=data.get("StandardizedPostalCode", ""),
            standardized_private_mailbox=data.get("StandardizedPrivateMailbox", ""),
            standardized_results=data.get("StandardizedResults", ""),
            standardized_state=data.get("StandardizedState", ""),
            standardized_state_name=data.get("StandardizedStateName", ""),
            standardized_suite=data.get("StandardizedSuite", ""),
            standardized_urbanization=data.get("StandardizedUrbanization", ""),
            dpv_foot_notes=data.get("DPVFootNotes", ""),
            move_return_code=data.get("MoveReturnCode", ""),
            plus4=data.get("Plus4", ""),
            private_mailbox=data.get("PrivateMailbox", ""),
            suite=data.get("Suite", ""),
        )
    
    # Setters
    def set_address_extras(self, address_extras):
        self.address_extras = address_extras

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_address_line_2(self, address_line_2):
        self.address_line_2 = address_line_2

    def set_address_type_code(self, address_type_code):
        self.address_type_code = address_type_code

    def set_base_melissa_address_key(self, base_melissa_address_key):
        self.base_melissa_address_key = base_melissa_address_key

    def set_carrier_route(self, carrier_route):
        self.carrier_route = carrier_route

    def set_city(self, city):
        self.city = city

    def set_city_abbreviation(self, city_abbreviation):
        self.city_abbreviation = city_abbreviation

    def set_company_name(self, company_name):
        self.company_name = company_name

    def set_country_code(self, country_code):
        self.country_code = country_code

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_delivery_indicator(self, delivery_indicator):
        self.delivery_indicator = delivery_indicator

    def set_delivery_point_check_digit(self, delivery_point_check_digit):
        self.delivery_point_check_digit = delivery_point_check_digit

    def set_delivery_point_code(self, delivery_point_code):
        self.delivery_point_code = delivery_point_code

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_move_effective_date(self, move_effective_date):
        self.move_effective_date = move_effective_date

    def set_move_type_code(self, move_type_code):
        self.move_type_code = move_type_code

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_state(self, state):
        self.state = state

    def set_state_name(self, state_name):
        self.state_name = state_name

    def set_urbanization(self, urbanization):
        self.urbanization = urbanization

    def set_name_first(self, name_first):
        self.name_first = name_first

    def set_name_full(self, name_full):
        self.name_full = name_full

    def set_name_last(self, name_last):
        self.name_last = name_last

    def set_name_middle(self, name_middle):
        self.name_middle = name_middle

    def set_name_prefix(self, name_prefix):
        self.name_prefix = name_prefix

    def set_name_suffix(self, name_suffix):
        self.name_suffix = name_suffix

    def set_original_address_line_1(self, original_address_line_1):
        self.original_address_line_1 = original_address_line_1

    def set_original_address_line_2(self, original_address_line_2):
        self.original_address_line_2 = original_address_line_2

    def set_original_city(self, original_city):
        self.original_city = original_city

    def set_original_city_abbreviation(self, original_city_abbreviation):
        self.original_city_abbreviation = original_city_abbreviation

    def set_original_country_code(self, original_country_code):
        self.original_country_code = original_country_code

    def set_original_plus4(self, original_plus4):
        self.original_plus4 = original_plus4

    def set_original_postal_code(self, original_postal_code):
        self.original_postal_code = original_postal_code

    def set_original_private_mailbox(self, original_private_mailbox):
        self.original_private_mailbox = original_private_mailbox

    def set_original_results(self, original_results):
        self.original_results = original_results

    def set_original_state(self, original_state):
        self.original_state = original_state

    def set_original_state_name(self, original_state_name):
        self.original_state_name = original_state_name

    def set_original_suite(self, original_suite):
        self.original_suite = original_suite

    def set_original_urbanization(self, original_urbanization):
        self.original_urbanization = original_urbanization

    def set_address_delivery_installation(self, address_delivery_installation):
        self.address_delivery_installation = address_delivery_installation

    def set_address_house_number(self, address_house_number):
        self.address_house_number = address_house_number

    def set_address_lock_box(self, address_lock_box):
        self.address_lock_box = address_lock_box

    def set_address_post_direction(self, address_post_direction):
        self.address_post_direction = address_post_direction

    def set_address_pre_direction(self, address_pre_direction):
        self.address_pre_direction = address_pre_direction

    def set_address_private_mailbox_name(self, address_private_mailbox_name):
        self.address_private_mailbox_name = address_private_mailbox_name

    def set_address_private_mailbox_range(self, address_private_mailbox_range):
        self.address_private_mailbox_range = address_private_mailbox_range

    def set_address_route_service(self, address_route_service):
        self.address_route_service = address_route_service

    def set_address_street_name(self, address_street_name):
        self.address_street_name = address_street_name

    def set_address_street_suffix(self, address_street_suffix):
        self.address_street_suffix = address_street_suffix

    def set_address_suite_name(self, address_suite_name):
        self.address_suite_name = address_suite_name

    def set_address_suite_number(self, address_suite_number):
        self.address_suite_number = address_suite_number

    def set_standardized_address_line_1(self, standardized_address_line_1):
        self.standardized_address_line_1 = standardized_address_line_1

    def set_standardized_address_line_2(self, standardized_address_line_2):
        self.standardized_address_line_2 = standardized_address_line_2

    def set_standardized_base_melissa_address_key(self, standardized_base_melissa_address_key):
        self.standardized_base_melissa_address_key = standardized_base_melissa_address_key

    def set_standardized_carrier_route(self, standardized_carrier_route):
        self.standardized_carrier_route = standardized_carrier_route

    def set_standardized_city(self, standardized_city):
        self.standardized_city = standardized_city

    def set_standardized_city_abbreviation(self, standardized_city_abbreviation):
        self.standardized_city_abbreviation = standardized_city_abbreviation

    def set_standardized_country_code(self, standardized_country_code):
        self.standardized_country_code = standardized_country_code

    def set_standardized_delivery_indicator(self, standardized_delivery_indicator):
        self.standardized_delivery_indicator = standardized_delivery_indicator

    def set_standardized_delivery_point_check_digit(self, standardized_delivery_point_check_digit):
        self.standardized_delivery_point_check_digit = standardized_delivery_point_check_digit

    def set_standardized_delivery_point_code(self, standardized_delivery_point_code):
        self.standardized_delivery_point_code = standardized_delivery_point_code

    def set_standardized_melissa_address_key(self, standardized_melissa_address_key):
        self.standardized_melissa_address_key = standardized_melissa_address_key

    def set_standardized_plus4(self, standardized_plus4):
        self.standardized_plus4 = standardized_plus4

    def set_standardized_postal_code(self, standardized_postal_code):
        self.standardized_postal_code = standardized_postal_code

    def set_standardized_private_mailbox(self, standardized_private_mailbox):
        self.standardized_private_mailbox = standardized_private_mailbox

    def set_standardized_results(self, standardized_results):
        self.standardized_results = standardized_results

    def set_standardized_state(self, standardized_state):
        self.standardized_state = standardized_state

    def set_standardized_state_name(self, standardized_state_name):
        self.standardized_state_name = standardized_state_name

    def set_standardized_suite(self, standardized_suite):
        self.standardized_suite = standardized_suite

    def set_standardized_urbanization(self, standardized_urbanization):
        self.standardized_urbanization = standardized_urbanization

    def set_dpv_foot_notes(self, dpv_foot_notes):
        self.dpv_foot_notes = dpv_foot_notes

    def set_move_return_code(self, move_return_code):
        self.move_return_code = move_return_code

    def set_plus4(self, plus4):
        self.plus4 = plus4

    def set_private_mailbox(self, private_mailbox):
        self.private_mailbox = private_mailbox

    def set_suite(self, suite):
        self.suite = suite

    # Getters
    def get_address_extras(self):
        return self.address_extras or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_address_line_2(self):
        return self.address_line_2 or ""

    def get_address_type_code(self):
        return self.address_type_code or ""

    def get_base_melissa_address_key(self):
        return self.base_melissa_address_key or ""

    def get_carrier_route(self):
        return self.carrier_route or ""

    def get_city(self):
        return self.city or ""

    def get_city_abbreviation(self):
        return self.city_abbreviation or ""

    def get_company_name(self):
        return self.company_name or ""

    def get_country_code(self):
        return self.country_code or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_delivery_indicator(self):
        return self.delivery_indicator or ""

    def get_delivery_point_check_digit(self):
        return self.delivery_point_check_digit or ""

    def get_delivery_point_code(self):
        return self.delivery_point_code or ""

    def get_melissa_address_key(self):
        return self.melissa_address_key or ""

    def get_move_effective_date(self):
        return self.move_effective_date or ""

    def get_move_type_code(self):
        return self.move_type_code or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_state(self):
        return self.state or ""

    def get_state_name(self):
        return self.state_name or ""

    def get_urbanization(self):
        return self.urbanization or ""

    def get_name_first(self):
        return self.name_first or ""

    def get_name_full(self):
        return self.name_full or ""

    def get_name_last(self):
        return self.name_last or ""

    def get_name_middle(self):
        return self.name_middle or ""

    def get_name_prefix(self):
        return self.name_prefix or ""

    def get_name_suffix(self):
        return self.name_suffix or ""

    def get_original_address_line_1(self):
        return self.original_address_line_1 or ""

    def get_original_address_line_2(self):
        return self.original_address_line_2 or ""

    def get_original_city(self):
        return self.original_city or ""

    def get_original_city_abbreviation(self):
        return self.original_city_abbreviation or ""

    def get_original_country_code(self):
        return self.original_country_code or ""

    def get_original_plus4(self):
        return self.original_plus4 or ""

    def get_original_postal_code(self):
        return self.original_postal_code or ""

    def get_original_private_mailbox(self):
        return self.original_private_mailbox or ""

    def get_original_results(self):
        return self.original_results or ""

    def get_original_state(self):
        return self.original_state or ""

    def get_original_state_name(self):
        return self.original_state_name or ""

    def get_original_suite(self):
        return self.original_suite or ""

    def get_original_urbanization(self):
        return self.original_urbanization or ""

    def get_address_delivery_installation(self):
        return self.address_delivery_installation or ""

    def get_address_house_number(self):
        return self.address_house_number or ""

    def get_address_lock_box(self):
        return self.address_lock_box or ""

    def get_address_post_direction(self):
        return self.address_post_direction or ""

    def get_address_pre_direction(self):
        return self.address_pre_direction or ""

    def get_address_private_mailbox_name(self):
        return self.address_private_mailbox_name or ""

    def get_address_private_mailbox_range(self):
        return self.address_private_mailbox_range or ""

    def get_address_route_service(self):
        return self.address_route_service or ""

    def get_address_street_name(self):
        return self.address_street_name or ""

    def get_address_street_suffix(self):
        return self.address_street_suffix or ""

    def get_address_suite_name(self):
        return self.address_suite_name or ""

    def get_address_suite_number(self):
        return self.address_suite_number or ""

    def get_standardized_address_line_1(self):
        return self.standardized_address_line_1 or ""

    def get_standardized_address_line_2(self):
        return self.standardized_address_line_2 or ""

    def get_standardized_base_melissa_address_key(self):
        return self.standardized_base_melissa_address_key or ""

    def get_standardized_carrier_route(self):
        return self.standardized_carrier_route or ""

    def get_standardized_city(self):
        return self.standardized_city or ""

    def get_standardized_city_abbreviation(self):
        return self.standardized_city_abbreviation or ""

    def get_standardized_country_code(self):
        return self.standardized_country_code or ""

    def get_standardized_delivery_indicator(self):
        return self.standardized_delivery_indicator or ""

    def get_standardized_delivery_point_check_digit(self):
        return self.standardized_delivery_point_check_digit or ""

    def get_standardized_delivery_point_code(self):
        return self.standardized_delivery_point_code or ""

    def get_standardized_melissa_address_key(self):
        return self.standardized_melissa_address_key or ""

    def get_standardized_plus4(self):
        return self.standardized_plus4 or ""

    def get_standardized_postal_code(self):
        return self.standardized_postal_code or ""

    def get_standardized_private_mailbox(self):
        return self.standardized_private_mailbox or ""

    def get_standardized_results(self):
        return self.standardized_results or ""

    def get_standardized_state(self):
        return self.standardized_state or ""

    def get_standardized_state_name(self):
        return self.standardized_state_name or ""

    def get_standardized_suite(self):
        return self.standardized_suite or ""

    def get_standardized_urbanization(self):
        return self.standardized_urbanization or ""

    def get_dpv_foot_notes(self):
        return self.dpv_foot_notes or ""

    def get_move_return_code(self):
        return self.move_return_code or ""

    def get_plus4(self):
        return self.plus4 or ""

    def get_private_mailbox(self):
        return self.private_mailbox or ""

    def get_suite(self):
        return self.suite or ""




    